@@ -3,6 +3,7 @@
 :class:`Property` objects can be defined on subclasses of
 :class:`~pydispatch.Dispatcher` to create instance attributes that act as events
 when their values change::
+
     from pydispatch import Dispatcher, Property
 
     class Foo(Dispatcher):
@@ -34,16 +35,24 @@ reason. If custom objects are used as values, they must be able to support
 equality checking. In most cases, this will be handled automatically.
 """
 
+import sys
 import weakref
 
 from pydispatch.utils import InformativeWVDict
 
+PY2 = sys.version_info < (3,)
+
 class Property(object):
     """Defined on the class level to create an observable attribute
 
     Args:
         default (Optional): If supplied, this will be the default value of the
             Property for all instances of the class. Otherwise ``None``
+
+    Attributes:
+        name (str): The name of the Property as defined in the class definition.
+            This will match the attribute name for the :class:`Dispatcher` instance.
+
     """
     def __init__(self, default=None):
         self._name = ''
@@ -83,8 +92,25 @@ class Property(object):
             return
         self.__storage[obj_id] = value
         self._on_change(obj, current, value)
-    def _on_change(self, obj, old, value):
-        obj.emit(self.name, obj, value, old=old, property=self)
+    def _on_change(self, obj, old, value, **kwargs):
+        """Called internally to emit changes from the instance object
+
+        The keyword arguments here will be passed to callbacks through the
+        instance object's :meth:`~Dispatcher.emit` method.
+
+        Keyword Args:
+            property: The :class:`Property` instance. This is useful if multiple
+                properties are bound to the same callback. The attribute name
+            keys (optional): If the :class:`Property` is a container type
+                (:class:`ListProperty` or :class:`DictProperty`), the changes
+                may be found here.
+                This is not implemented for nested containers and will only be
+                available for operations that do not alter the size of the
+                container.
+
+        """
+        kwargs['property'] = self
+        obj.emit(self.name, obj, value, old=old, **kwargs)
     def __repr__(self):
         return '<{}: {}>'.format(self.__class__, self)
     def __str__(self):
@@ -161,13 +187,11 @@ class Observable(object):
     def _emit_change(self, **kwargs):
         if not self._init_complete:
             return
-        if 'value' not in kwargs:
-            kwargs['value'] = self
         p = self.parent_observable
         if p is not None:
-            p._emit_change(**kwargs)
+            p._emit_change()
             return
-        self.property._on_change(self.obj, None, kwargs['value'])
+        self.property._on_change(self.obj, None, self, **kwargs)
 
 class ObservableList(list, Observable):
     """A ``list`` object that tracks changes to its contents
@@ -184,14 +208,24 @@ class ObservableList(list, Observable):
     def __setitem__(self, key, item):
         item = self._build_observable(item)
         super(ObservableList, self).__setitem__(key, item)
-        self._emit_change()
+        self._emit_change(keys=[key])
     def __delitem__(self, key):
         super(ObservableList, self).__delitem__(key)
         self._emit_change()
+    if PY2:
+        def __setslice__(self, *args):
+            super(ObservableList, self).__setslice__(*args)
+            self._emit_change()
+        def __delslice__(self, *args):
+            super(ObservableList, self).__delslice__(*args)
+            self._emit_change()
+    if hasattr(list, 'clear'):
+        def clear(self):
+            super(ObservableList, self).clear()
+            self._emit_change()
     def __iadd__(self, other):
         other = self._build_observable(other)
         self.extend(other)
-        self._emit_change()
         return self
     def append(self, item):
         item = self._build_observable(item)
@@ -224,18 +258,22 @@ class ObservableDict(dict, Observable):
     def __setitem__(self, key, item):
         item = self._build_observable(item)
         super(ObservableDict, self).__setitem__(key, item)
-        self._emit_change()
+        self._emit_change(keys=[key])
     def __delitem__(self, key):
         super(ObservableDict, self).__delitem__(key)
         self._emit_change()
     def update(self, other):
         init = self._init_complete
         self._init_complete = False
+        keys = set(other.keys()) - set(self.keys())
         for key, val in other.items():
+            if key not in keys and self[key] == val:
+                continue
             self[key] = val
+            keys.add(key)
         if init:
             self._init_complete = True
-        self._emit_change()
+        self._emit_change(keys=list(keys))
     def clear(self):
         super(ObservableDict, self).clear()
         self._emit_change()
